/*
 * commonFunctions.js 
 * Copyright  2007 Tommi Rautava
 * 
 * This file is part of Popomungo.
 *
 * Popomungo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Popomungo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

///////////////////////////////////////////////////////////////////////
// Popomungo common functions
///////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////
function pm_ExtensionVersion() {
	var extensionsManager = Components.classes["@mozilla.org/extensions/manager;1"]
		.getService(Components.interfaces.nsIExtensionManager);
	
	var popomungo = extensionsManager.getItemForID("{9935ef86-61cd-4f48-a057-ab2c5172031d}");
	
	return popomungo.version;	
};


///////////////////////////////////////////////////////////////////////
function pm_InsertRowAfter(key, value, hover, afterRow) {
	var doc = afterRow.ownerDocument;
	var newRow = doc.createElement('tr');
	var newData1 = doc.createElement('td');
	var newData2 = doc.createElement('td');

	// FIXME Replace innerHTML manipulation.
	newData1.innerHTML = '<b class="popomungo_added">'+ key +':</b>';
	if (hover) {
		newData2.innerHTML = '<span class="popomungo_added" title="'+ 
			hover +'">'+ value +'</span>';
	} else {
		newData2.innerHTML = '<span class="popomungo_added">'+ 
			value +'</span>';
	}

	newRow.appendChild(newData1);
	newRow.appendChild(newData2);

	afterRow.parentNode.insertBefore(newRow, afterRow.nextSibling);

	return newRow;
};


///////////////////////////////////////////////////////////////////////
function pm_RunOnAllElements(aDocument, aXpath, aFunction) {
	try {
		var elements = pm_XPathOrderedSnapshot(aXpath, aDocument);
		
		pm_Logger.debug('elements='+ elements.snapshotLength);
	
		var i;
		// Loop through all items.
		for (i = elements.snapshotLength - 1; i >= 0; i--) {
			var elem = elements.snapshotItem(i);
			//pm_Logger.debug(i +': '+ elem.innerHTML);
			aFunction(elem);
		}
	}
	catch (err) {
		pm_Logger.logError(err);
	}
};


///////////////////////////////////////////////////////////////////////
function pm_CreateTableRow(doc, arg) {
	var tr = doc.createElement('tr');
	
	pm_Logger.debug('arg='+ arg);
	
	if (arg.length) {
		for (var i=0; i<arg.length; i++) {
			var td = doc.createElement('td');
			td.appendChild(arg[i]);
			tr.appendChild(td);
		}
	} else {
		var td = doc.createElement('td');
		td.appendChild(arg);
		tr.appendChild(td);
	}
	
	//pm_Logger.debug('tr.innerHTML='+ tr.innerHTML);
	
	return tr;
};


///////////////////////////////////////////////////////////////////////
function pm_CreateAnchor(doc, href, arg) {
	var anchor = doc.createElement('a');
	anchor.href = href;

	if (arg.length) {
		for (var i=0; i<arg.length; i++) {
			anchor.appendChild(arg[i]);
		}
	} else {
		anchor.appendChild(arg);
	}

	return anchor;
};


/*
 * This function is based on the HTML parser example that was published on 
 * the Greasemonkey blog 4th December 2005 by boogs 
 * (Aaron Boodman <boogs@youngpup.net>).
 * 
 * The blog article:
 * http://www.greasespot.net/2005/12/workarounds-for-missing-xmlhttprequest.html
 * 
 * The example script:
 * http://youngpup.net/userscripts/htmlparserexample.user.js
 */ 
function pm_ConvertResponseTextIntoIFrame(doc, responseText, callback) 
{
	try {
		// create an IFRAME to write the document into. the iframe must be added
		// to the document and rendered (eg display != none) to be property 
		// initialized.
		var iframe = doc.createElement('iframe');
		iframe.style.visibility = 'hidden';
		iframe.style.position = 'absolute';
		iframe.id = 'popomungo_iframe';
		doc.body.appendChild(iframe);
		
		// give it a URL so that it will create a .contentDocument property. Make
		// the URL be the current page's URL so that we can communicate with it.
		// Otherwise, same-origin policy would prevent us.
		iframe.contentWindow.location.href = doc.location.href;
		
		// write the received content into the document
		iframe.contentDocument.open('text/html');
		iframe.contentDocument.write(responseText);
		
		iframe.contentDocument.addEventListener(
			'DOMContentLoaded', 
			function() {
				callback(iframe.contentDocument);
				iframe.parentNode.removeChild(iframe);
			},
			false
		);
		
		iframe.contentDocument.close();
	}
	catch (err) {
		pm_Logger.logError(err);
	}
	
	return true;
};


///////////////////////////////////////////////////////////////////////
function pm_Trim(s) {
	return s.replace(/^\s+|\s+$/g, '');
}


///////////////////////////////////////////////////////////////////////
function pm_XPathFirstNode(aXPath, aDocument) {
	return aDocument.evaluate(aXPath, aDocument, null, 
		XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
}


///////////////////////////////////////////////////////////////////////
function pm_XPathOrderedSnapshot(aXPath, aDocument) {
	return aDocument.evaluate(aXPath, aDocument, null, 
		XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
}

///////////////////////////////////////////////////////////////////////
function pm_AddClassName(node, className) {
	if (node.className) {
		node.className += " "+ className;
	}
	else {
		node.className = className;
	}
}

///////////////////////////////////////////////////////////////////////
function pm_IsOfClass(node, className) {
	var classNames = node.className.split(/s+/);

	for (var i=classNames.length; i >= 0; i--) {
		if (classNames[i] == className) return true;
	}
	
	return false;
}

///////////////////////////////////////////////////////////////////////
function pm_NextSibling(node, tagName) {
	tagName = tagName.toUpperCase();
	node = node.nextSibling;
	
	while (node) {
		if (node.tagName == tagName) {
			return node;
		}
		
		node = node.nextSibling;
	}
	
	return node;
}

///////////////////////////////////////////////////////////////////////
// EOF
///////////////////////////////////////////////////////////////////////
